import express,{Router, Request, Response } from 'express';
import {Task} from '../models/Task';
import {myDataSource} from '../app-data-source';
import { User } from '../models/User';
import { Member } from '../models/Member';
import { In } from 'typeorm';
import { DateTimeTask } from '../models/DateTimeTask';
import { auth } from '../helpers';
const router:Router = express.Router();
const dataSource = myDataSource;


/**get all tasks */
router.get('/', async(req:Request, res:Response):Promise<Response> => {
    const tasks = await dataSource
.getRepository(Task)
.find({
    relations:{
        executor:true,
        author:true,
    }})
    return res.send({tasks})
})


/**create new task */
router.post('/', auth, async(req:Request, res:Response):Promise<Response>=>{
    const {user,title,description,project,dateTimeTaskArray, dateTimeDeadLine,priority} = req.body;
    const newTask = new Task();
    const dateTimeArray=[];
    if(dateTimeTaskArray && dateTimeTaskArray.length>0){
        for (let dateTimeStartDue of dateTimeTaskArray){
            const newDateTimeTask = new DateTimeTask();
            newDateTimeTask.dateTimeStart = dateTimeStartDue.dateTimeStart;
            newDateTimeTask.dateTimeDue = dateTimeStartDue.dateTimeDue;
            await newDateTimeTask.save();
            dateTimeArray.push(newDateTimeTask);
        }
        newTask.dateTimeTasks=dateTimeArray;
    }
    newTask.title = title;
    newTask.description = description;
    newTask.project = project;
    newTask.dateTimeDeadLine=dateTimeDeadLine;
    newTask.author= user;
    newTask.priority = priority;
    await newTask.save(); 
    return res.send({newTask});
})


/**check tasks of specific user by userID */
router.get('/user/:userId', async (req: Request, res: Response):Promise<Response>=>{   
    const userId = req.params.userId;
    const tasks = await dataSource
    .getRepository(Task)
    .find({
        relations:{
            executor:true,
            author:true,
        },
    where:[
        {
        executor:{
            id:userId
        }
    },
    {
        author:{
            id:userId
        }
    },
]
})
    return res.send({tasks})
})


/**check tasks of current user where he is author or executor, search by id*/
router.get('/my',auth, async (req: Request, res: Response):Promise<Response>=>{
    const token = req.get('Authorization');
    const user = req.body.user
    const tasks = await dataSource
    .getRepository(Task)
    .find({
        relations:{
            executor:true,
            author:true,
        },
    where:[
        {
        executor:{
            id:user.id
        }
    },
    {
        author:{
            id:user.id
        }
    },
]
})
    return res.send({tasks})
})


/**check tasks of projects in which current user is involved, search by token*/
router.get('/related', auth,async (req: Request, res: Response):Promise<Response>=>{
    const user = req.body.user
    const rawMembership = await dataSource
    .createQueryBuilder()
    .select("member")
    .from(Member, "member")
    .leftJoinAndSelect('member.user', 'user' )
    .leftJoinAndSelect('member.project', 'project' )
    .leftJoinAndSelect('project.tasks', 'task' )
    .where("member.userId = :id", { id: user.id })
    .getMany()
     let projectIds:any[] = []
    if (rawMembership.length>0){
        for (let member of rawMembership){
            const projectId = member?.project?.id
            projectIds.push(projectId)
        }
    }
    let tasks :Task[]= []
    const projectsIdArray = [...new Set(projectIds)]; 
    const searchByProjectsIdArray:object[] = []
    for (let projectId of projectsIdArray){
        searchByProjectsIdArray.push({id:projectId})
    }
    if (projectIds.length>0){
    tasks = await dataSource
   .getRepository(Task)
.find({
        relations:{
        author:true,
        executor:true,
        project:true    
        },
        where:{
        project:
            {id:In(
                projectsIdArray
            )}
        }
    })
}
    return res.send({tasks})
})


/**delete of task by task id */
router.delete('/:taskId',async (req: Request, res: Response):Promise<Response>=>{
    const taskId = req.params.taskId;
    await myDataSource
    .createQueryBuilder()
    .delete()
    .from(Task)
    .where("id = :id", { id: taskId })
    .execute()
    return res.send({message: 'Task deleted successfully'})

})


/**change of task  by task id */
router.put('/',auth,async(req:Request, res:Response)=> {
    const {user,id,title,description,project,dateTimeTaskArray,executor,accomplish,dateTimeDeadLine,priority} = req.body;
    const task = await dataSource
    .createQueryBuilder()
    .select("task")
    .from(Task, "task")
    .where("task.id = :id", { id })
    .getOne()
    if (!task) return res.status(404).send({Message:'task not found'})
    const dateTimeArray=[];
    if(dateTimeTaskArray && dateTimeTaskArray.length>0){
        for (let dateTimeStartDue of dateTimeTaskArray){
            const newDateTimeTask = new DateTimeTask();
            newDateTimeTask.dateTimeStart = dateTimeStartDue.dateTimeStart;
            newDateTimeTask.dateTimeDue = dateTimeStartDue.dateTimeDue;
            await newDateTimeTask.save();
            dateTimeArray.push(newDateTimeTask);
        }
        task.dateTimeTasks=dateTimeArray;
    }
    task.title= title
    task.description= description
    task.project= project
    task.dateTimeDeadLine=dateTimeDeadLine;
    task.author=user
    task.executor=executor
    task.accomplish= accomplish
    task.priority= priority    
    await task.save()
    res.send({message:'update task successfully'})
})


/** search all tasks on given projects: project Id array[] */
router.post('/project',async (req: Request, res: Response):Promise<Response>=>{
    let projectArray :string[]= req.body;
    if (projectArray.length===0) {
        const rawTasks = await
        dataSource
       .getRepository(Task)
        .find({
            relations:{
                executor:true,
                author:true,
                project:true
            }
        })
    const tasks:object[]= rawTasks.filter(task=>task.project===null)
       return res.send({tasks})
    } 
    const tasks = await
     dataSource
    .getRepository(Task)
    .createQueryBuilder('task')
    .innerJoinAndSelect( 'task.project', 'project')
    .where('task.project IN(:...projects)', {projects:projectArray})
    .getMany()
    return res.send({tasks})
}) 





export default router;
